/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of version 3 of the GNU Lesser General Public License as published
 * by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Pete Woods <pete.woods@canonical.com>
 */

#include <stdexcept>

#include <libusermetricsinput/MetricImpl.h>
#include <libusermetricsinput/MetricUpdateImpl.h>
#include <libusermetricscommon/DBusPaths.h>
#include <libusermetricscommon/UserDataInterface.h>
#include <libusermetricscommon/DataSetInterface.h>

#include <QtDBus/QtDBus>

using namespace std;
using namespace UserMetricsCommon;
using namespace UserMetricsInput;

MetricImpl::MetricImpl(const QString &dataSourceId, const QString &formatString,
		const QDBusConnection &dbusConnection, QObject *parent) :
		Metric(parent), m_dbusConnection(dbusConnection), m_interface(
				DBusPaths::serviceName(), DBusPaths::userMetrics(),
				dbusConnection), m_dataSourceId(dataSourceId), m_formatString(
				formatString) {
}

MetricImpl::~MetricImpl() {
}

QDBusObjectPath MetricImpl::createDataSet(const QString &usernameIn) {
	QString username;
	if (usernameIn.isEmpty()) {
		username = QString::fromUtf8(qgetenv("USER"));
	} else {
		username = usernameIn;
	}

	QDBusObjectPath userDataPath(m_interface.createUserData(username));

	com::canonical::usermetrics::UserData userDataInterface(
			DBusPaths::serviceName(), userDataPath.path(), m_dbusConnection);
	if (!userDataInterface.isValid()) {
		throw logic_error("user data interface invalid");
	}

	return userDataInterface.createDataSet(m_dataSourceId);
}

MetricUpdate * MetricImpl::update(const QString &username) {
	QDBusObjectPath dataSetPath(createDataSet(username));

	return new MetricUpdateImpl(dataSetPath.path(), m_dbusConnection);
}

void MetricImpl::update(double value, const QString &username) {
	QDBusObjectPath dataSetPath(createDataSet(username));

	com::canonical::usermetrics::DataSet dataSetInterface(
			DBusPaths::serviceName(), dataSetPath.path(), m_dbusConnection);

	dataSetInterface.update(QVariantList() << value);
}

void MetricImpl::increment(double amount, const QString &username) {
	QDBusObjectPath dataSetPath(createDataSet(username));

	com::canonical::usermetrics::DataSet dataSetInterface(
			DBusPaths::serviceName(), dataSetPath.path(), m_dbusConnection);

	dataSetInterface.increment(amount);
}
